@@ -1,15 +1,17 @@
 /// Tools for placing all information about open files into one place
 use crate::editor::{get_absolute_path, Editor, FileType};
+use crate::pty::Pty;
 use crate::Loc;
 use kaolinite::Document;
 use kaolinite::Size;
 use std::ops::Range;
+use std::sync::{Arc, Mutex};
 use synoptic::Highlighter;
 
 pub type Span = Vec<(Vec<usize>, Range<usize>, Range<usize>)>;
 
 // File split structure
-#[derive(Debug, Clone)]
+#[derive(Debug)]
 pub enum FileLayout {
     /// Side-by-side documents (with proportions)
     SideBySide(Vec<(FileLayout, f64)>),
@@ -21,6 +23,8 @@ pub enum FileLayout {
     None,
     /// Representing a file tree
     FileTree,
+    /// Representing a terminal
+    Terminal(Arc<Mutex<Pty>>),
 }
 
 impl Default for FileLayout {
@@ -40,8 +44,8 @@ impl FileLayout {
     pub fn span(&self, idx: Vec<usize>, size: Size, at: Loc) -> Span {
         match self {
             Self::None => vec![],
-            // Atom and file trees: stretch from starting position through to end of their containers
-            Self::Atom(_, _) | Self::FileTree => {
+            // Atom file trees and terminals: stretch from starting position through to end of their containers
+            Self::Atom(_, _) | Self::FileTree | Self::Terminal(_) => {
                 vec![(idx, at.y..at.y + size.h, at.x..at.x + size.w)]
             }
             // SideBySide: distributes available container space to each sub-layout
@@ -157,7 +161,7 @@ impl FileLayout {
     /// Work out how many files are currently open
     pub fn len(&self) -> usize {
         match self {
-            Self::None | Self::FileTree => 0,
+            Self::None | Self::FileTree | Self::Terminal(_) => 0,
             Self::Atom(containers, _) => containers.len(),
             Self::SideBySide(layouts) => layouts.iter().map(|(layout, _)| layout.len()).sum(),
             Self::TopToBottom(layouts) => layouts.iter().map(|(layout, _)| layout.len()).sum(),
@@ -167,7 +171,7 @@ impl FileLayout {
     /// Work out how many atoms are currently open
     pub fn n_atoms(&self) -> usize {
         match self {
-            Self::None | Self::FileTree => 0,
+            Self::None | Self::FileTree | Self::Terminal(_) => 0,
             Self::Atom(_, _) => 1,
             Self::SideBySide(layouts) => layouts.iter().map(|(layout, _)| layout.n_atoms()).sum(),
             Self::TopToBottom(layouts) => layouts.iter().map(|(layout, _)| layout.n_atoms()).sum(),
@@ -177,7 +181,7 @@ impl FileLayout {
     /// Find a file container location from it's path
     pub fn find(&self, idx: Vec<usize>, path: &str) -> Option<(Vec<usize>, usize)> {
         match self {
-            Self::None | Self::FileTree => None,
+            Self::None | Self::FileTree | Self::Terminal(_) => None,
             Self::Atom(containers, _) => {
                 // Scan this atom for any documents
                 for (ptr, container) in containers.iter().enumerate() {
@@ -207,7 +211,7 @@ impl FileLayout {
     /// Get the `FileLayout` at a certain index
     pub fn get_raw(&self, mut idx: Vec<usize>) -> Option<&FileLayout> {
         match self {
-            Self::None | Self::Atom(_, _) | Self::FileTree => Some(self),
+            Self::None | Self::Atom(_, _) | Self::FileTree | Self::Terminal(_) => Some(self),
             Self::SideBySide(layouts) => {
                 if idx.is_empty() {
                     Some(self)
@@ -233,7 +237,7 @@ impl FileLayout {
             Some(self)
         } else {
             match self {
-                Self::None | Self::Atom(_, _) | Self::FileTree => Some(self),
+                Self::None | Self::Atom(_, _) | Self::FileTree | Self::Terminal(_) => Some(self),
                 Self::SideBySide(layouts) => {
                     let subidx = idx.remove(0);
                     layouts.get_mut(subidx)?.0.get_raw_mut(idx)
@@ -249,7 +253,7 @@ impl FileLayout {
     /// Get the `FileLayout` at a certain index
     pub fn set(&mut self, mut idx: Vec<usize>, fl: FileLayout) {
         match self {
-            Self::None | Self::Atom(_, _) | Self::FileTree => *self = fl,
+            Self::None | Self::Atom(_, _) | Self::FileTree | Self::Terminal(_) => *self = fl,
             Self::SideBySide(layouts) | Self::TopToBottom(layouts) => {
                 if idx.is_empty() {
                     *self = fl;
@@ -264,7 +268,7 @@ impl FileLayout {
     /// Given an index, find the file containers in the tree
     pub fn get_atom(&self, mut idx: Vec<usize>) -> Option<(&[FileContainer], usize)> {
         match self {
-            Self::None | Self::FileTree => None,
+            Self::None | Self::FileTree | Self::Terminal(_) => None,
             Self::Atom(containers, ptr) => Some((containers, *ptr)),
             Self::SideBySide(layouts) => {
                 let subidx = idx.remove(0);
@@ -283,7 +287,7 @@ impl FileLayout {
         mut idx: Vec<usize>,
     ) -> Option<(&mut Vec<FileContainer>, &mut usize)> {
         match self {
-            Self::None | Self::FileTree => None,
+            Self::None | Self::FileTree | Self::Terminal(_) => None,
             Self::Atom(ref mut containers, ref mut ptr) => Some((containers, ptr)),
             Self::SideBySide(layouts) => {
                 let subidx = idx.remove(0);
@@ -316,7 +320,7 @@ impl FileLayout {
     /// In the currently active atom, move to a different document
     pub fn move_to(&mut self, mut idx: Vec<usize>, ptr: usize) {
         match self {
-            Self::None | Self::FileTree => (),
+            Self::None | Self::FileTree | Self::Terminal(_) => (),
             Self::Atom(_, ref mut old_ptr) => *old_ptr = ptr,
             Self::SideBySide(layouts) | Self::TopToBottom(layouts) => {
                 let subidx = idx.remove(0);
@@ -341,7 +345,8 @@ impl FileLayout {
             let multi = self.get_raw_mut(redundant_idx.clone());
             if let Some(layout) = multi {
                 if let Self::SideBySide(layouts) | Self::TopToBottom(layouts) = layout {
-                    *layout = layouts[0].0.clone();
+                    let retain = std::mem::take(&mut layouts[0].0);
+                    *layout = retain;
                     if idx.starts_with(&redundant_idx) {
                         idx.remove(redundant_idx.len());
                         return idx;
@@ -361,7 +366,9 @@ impl FileLayout {
             // Determine behaviour based on parent
             if let Some(parent) = self.get_raw_mut(at_parent) {
                 match parent {
-                    Self::None | Self::Atom(_, _) | Self::FileTree => unreachable!(),
+                    Self::None | Self::Atom(_, _) | Self::FileTree | Self::Terminal(_) => {
+                        unreachable!()
+                    }
                     Self::SideBySide(layouts) | Self::TopToBottom(layouts) => {
                         // Get the proportion of what we're removing
                         let removed_prop = layouts[within_parent].1;
@@ -385,7 +392,7 @@ impl FileLayout {
     /// Traverse the tree and return a list of indices to empty atoms
     pub fn empty_atoms(&self, at: Vec<usize>) -> Option<Vec<usize>> {
         match self {
-            Self::None | Self::FileTree => None,
+            Self::None | Self::FileTree | Self::Terminal(_) => None,
             Self::Atom(fcs, _) => {
                 if fcs.is_empty() {
                     Some(at)
@@ -413,7 +420,7 @@ impl FileLayout {
     /// Traverse the tree and return a list of indices to redundant sidebyside/toptobottom
     pub fn redundant_multis(&self, at: Vec<usize>) -> Option<Vec<usize>> {
         match self {
-            Self::None | Self::FileTree | Self::Atom(_, _) => None,
+            Self::None | Self::FileTree | Self::Atom(_, _) | Self::Terminal(_) => None,
             Self::SideBySide(layouts) | Self::TopToBottom(layouts) => {
                 if layouts.len() == 1 {
                     Some(at)
@@ -431,6 +438,30 @@ impl FileLayout {
         }
     }
 
+    /// Traverse the tree and return a list of indices to empty atoms
+    pub fn terminal_rerender(&mut self) -> bool {
+        match self {
+            Self::None | Self::FileTree | Self::Atom(_, _) => false,
+            Self::Terminal(term) => {
+                let mut term = term.lock().unwrap();
+                if term.force_rerender {
+                    term.force_rerender = false;
+                    true
+                } else {
+                    false
+                }
+            }
+            Self::SideBySide(layouts) | Self::TopToBottom(layouts) => {
+                for layout in layouts.iter_mut() {
+                    if layout.0.terminal_rerender() {
+                        return true;
+                    }
+                }
+                false
+            }
+        }
+    }
+
     /// Find a new pointer position when something is removed
     pub fn new_pointer_position(&self, old: &[usize]) -> Vec<usize> {
         // Zoom out until a sidebyside or toptobottom is found
@@ -453,12 +484,16 @@ impl FileLayout {
     /// Open a split above the current pointer
     pub fn open_up(&mut self, at: Vec<usize>, fl: FileLayout) -> Vec<usize> {
         let mut new_ptr = at.clone();
-        if let Some(old_fl) = self.get_raw(at.clone()) {
+        if let Some(old_fl) = self.get_raw_mut(at.clone()) {
             let new_fl = match old_fl {
                 Self::None => fl,
-                Self::Atom(_, _) | Self::SideBySide(_) | Self::TopToBottom(_) => {
+                Self::Atom(_, _)
+                | Self::SideBySide(_)
+                | Self::TopToBottom(_)
+                | Self::Terminal(_) => {
                     new_ptr.push(0);
-                    Self::TopToBottom(vec![(fl, 0.5), (old_fl.clone(), 0.5)])
+                    let old_fl = std::mem::replace(old_fl, FileLayout::None);
+                    Self::TopToBottom(vec![(fl, 0.5), (old_fl, 0.5)])
                 }
                 Self::FileTree => return at,
             };
@@ -470,12 +505,16 @@ impl FileLayout {
     /// Open a split below the current pointer
     pub fn open_down(&mut self, at: Vec<usize>, fl: FileLayout) -> Vec<usize> {
         let mut new_ptr = at.clone();
-        if let Some(old_fl) = self.get_raw(at.clone()) {
+        if let Some(old_fl) = self.get_raw_mut(at.clone()) {
             let new_fl = match old_fl {
                 Self::None => fl,
-                Self::Atom(_, _) | Self::SideBySide(_) | Self::TopToBottom(_) => {
+                Self::Atom(_, _)
+                | Self::SideBySide(_)
+                | Self::TopToBottom(_)
+                | Self::Terminal(_) => {
                     new_ptr.push(1);
-                    Self::TopToBottom(vec![(old_fl.clone(), 0.5), (fl, 0.5)])
+                    let old_fl = std::mem::replace(old_fl, FileLayout::None);
+                    Self::TopToBottom(vec![(old_fl, 0.5), (fl, 0.5)])
                 }
                 Self::FileTree => return at,
             };
@@ -487,12 +526,16 @@ impl FileLayout {
     /// Open a split to the left of the current pointer
     pub fn open_left(&mut self, at: Vec<usize>, fl: FileLayout) -> Vec<usize> {
         let mut new_ptr = at.clone();
-        if let Some(old_fl) = self.get_raw(at.clone()) {
+        if let Some(old_fl) = self.get_raw_mut(at.clone()) {
             let new_fl = match old_fl {
                 Self::None => fl,
-                Self::Atom(_, _) | Self::SideBySide(_) | Self::TopToBottom(_) => {
+                Self::Atom(_, _)
+                | Self::SideBySide(_)
+                | Self::TopToBottom(_)
+                | Self::Terminal(_) => {
                     new_ptr.push(0);
-                    Self::SideBySide(vec![(fl, 0.5), (old_fl.clone(), 0.5)])
+                    let old_fl = std::mem::replace(old_fl, FileLayout::None);
+                    Self::SideBySide(vec![(fl, 0.5), (old_fl, 0.5)])
                 }
                 Self::FileTree => return at,
             };
@@ -504,12 +547,16 @@ impl FileLayout {
     /// Open a split to the right of the current pointer
     pub fn open_right(&mut self, at: Vec<usize>, fl: FileLayout) -> Vec<usize> {
         let mut new_ptr = at.clone();
-        if let Some(old_fl) = self.get_raw(at.clone()) {
+        if let Some(old_fl) = self.get_raw_mut(at.clone()) {
             let new_fl = match old_fl {
                 Self::None => fl,
-                Self::Atom(_, _) | Self::SideBySide(_) | Self::TopToBottom(_) => {
+                Self::Atom(_, _)
+                | Self::SideBySide(_)
+                | Self::TopToBottom(_)
+                | Self::Terminal(_) => {
                     new_ptr.push(1);
-                    Self::SideBySide(vec![(old_fl.clone(), 0.5), (fl, 0.5)])
+                    let old_fl = std::mem::replace(old_fl, FileLayout::None);
+                    Self::SideBySide(vec![(old_fl, 0.5), (fl, 0.5)])
                 }
                 Self::FileTree => return at,
             };
